@@ -7,17 +7,14 @@ import logging
 import concurrent.futures
 
 import can
-can.rc['interface'] = 'socketcan_native'
-from can.interfaces.interface import Bus
-can_interface = 'vcan0'
 
 
 def producer(id):
     """:param id: Spam the bus with messages including the data id."""
 
-    bus = Bus(can_interface)
+    bus = can.interface.Bus(bustype='socketcan', channel='vcan0')
     for i in range(16):
-        msg = can.Message(arbitration_id=0x0cf02200, data=[id, i, 0, 1, 3, 1, 4, 1])
+        msg = can.Message(arbitration_id=0x0cf02200+id, data=[id, i, 0, 1, 3, 1, 4, 1])
         bus.send(msg)
     # TODO Issue #3: Need to keep running to ensure the writing threads stay alive. ?
     time.sleep(2)
